"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.handleError = exports.handleResponse = exports.handleRequest = void 0;

var _react = _interopRequireDefault(require("react"));

var _axios = _interopRequireDefault(require("axios"));

var _q3UiForms = require("q3-ui-forms");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const extractResponseMeta = payload => {
  const {
    config,
    data,
    headers,
    status
  } = 'response' in payload ? payload.response : payload;
  const output = {
    data,
    headers,
    status
  };
  if (config) Object.assign(output, {
    method: config.method,
    url: config.url
  });
  return output;
};

const queryStorage = key => {
  try {
    return JSON.parse(localStorage.getItem(key));
  } catch (e) {
    return null;
  }
};

const handleRequest = request => {
  const cache = queryStorage(request.baseURL + request.url);
  return cache ? Object.assign(request.headers.common, {
    'If-Match': cache.ETag,
    'If-Unmodified-Since': cache['Last-Modified']
  }) : null;
};

exports.handleRequest = handleRequest;

const handleResponse = d => {
  const {
    method,
    status,
    data
  } = extractResponseMeta(d);

  const startsWith = num => String(status).startsWith(String(num));

  return {
    notify(noti) {
      if (method !== 'get' && startsWith(2) && data.message) {
        noti.onSuccess(data.message);
      } else if (startsWith(4) || startsWith(5)) {
        noti.onFail(data.message);
      }

      return this;
    },

    set() {
      return this;
    }

  };
};

exports.handleResponse = handleResponse;

const handleError = e => {
  const {
    data,
    status
  } = extractResponseMeta(e);
  return {
    notify(noti) {
      if (status === 412) {
        noti.onFail('Race condition detected (412)');
      } else if (data && data.message && status !== 404) {
        noti.onFail(data.message);
      }

      return this;
    },

    refresh() {
      return Promise.reject(e);
    }

  };
};

exports.handleError = handleError;

var _default = () => {
  const [loading, setLoading] = _react.default.useState(false);

  const noti = (0, _q3UiForms.useNotification)();

  const onRequest = request => {
    setLoading(true);
    return request;
  };

  const onResponse = response => {
    setLoading(false);
    handleResponse(response).notify(noti).set();
    return response;
  };

  const onResponseError = error => {
    setLoading(false);

    try {
      return handleError(error).notify(noti).refresh(error);
    } catch (e) {
      return Promise.reject(error);
    }
  };

  _react.default.useEffect(() => {
    _axios.default.interceptors.request.use(onRequest, error => {
      setLoading(false);
      return error;
    });

    _axios.default.interceptors.response.use(onResponse, onResponseError);
  }, [loading]);

  return loading;
};

exports.default = _default;